#include "p33fj12gp202.h"

#define FCY 36864000
#include <Libpic30.h> //  after you defined FCY in Hz

#define LED _RB5  // vystup na led na doske PIC
#define SS _RB15   // slave select na led bar
#define SS_Adc _RA0 // slave select pre prevodnik

 unsigned int cnt = 0;

 unsigned int L_out,R_out;
 unsigned int L_led,R_led;

// koeficienty maju len 8 bitov MSB na vstupu, namiesto 16b. Toto avsak nevadi,
// nielenze nevadi ale by bolo aj zbytocne,
// pretoze prva hodnota (najmensia) na ktory led graf reaguje bude okolo 1000,
// a maximalna odchylka od pozadovanych hodnot je velmi mala (vid graf v excelu)

// na 8 bitovy vstup a 16b vystup na jeden kanal to potom cini 512B,
// na 16 bitovy vstup a 16b vystup by to bolo na jeden kanal 128KB


// hodnoty su ulozene vo flash memorii, nie v RAM
 unsigned int const coeff_L[128] = // deklaracia nelinearnej prevodnej charakteristyky pre zrychlenie programu
{0xFFFF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF,
0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF, 0xF7FF,
0xF7FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF,
0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF, 0xF3FF,
0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF,
0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF, 0xF1FF, 0xF0FF, 0xF0FF,
0xF0FF, 0xF0FF, 0xF0FF, 0xF0FF, 0xF0FF, 0xF0FF, 0xF0FF, 0xF0FF,
0xF0FF, 0xF0FF, 0xF07F, 0xF07F, 0xF07F, 0xF07F, 0xF07F, 0xF07F,
0xF07F, 0xF07F, 0xF07F, 0xF07F, 0xF07F, 0xF03F, 0xF03F, 0xF03F,
0xF03F, 0xF03F, 0xF03F, 0xF03F, 0xF03F, 0xF03F, 0xF03F, 0xF03F,
0xF01F, 0xF01F, 0xF01F, 0xF01F, 0xF01F, 0xF01F, 0xF01F, 0xF01F,
0xF01F, 0xF01F, 0xF00F, 0xF00F, 0xF00F, 0xF00F, 0xF00F, 0xF00F,
0xF00F, 0xF00F, 0xF00F, 0xF007, 0xF007, 0xF007, 0xF007, 0xF007,
0xF007, 0xF007, 0xF007, 0xF007, 0xF003, 0xF003, 0xF003, 0xF003,
0xF003, 0xF003, 0xF003, 0xF003, 0xF003, 0xF001, 0xF001, 0xF001,
0xF001, 0xF001, 0xF001, 0xF001, 0xF001, 0xF001, 0xF001, 0xF000};

unsigned int const coeff_R[128] = // deklaracia nelinearnej prevodnej charakteristyky pre zrychlenie programu
{0xFFFF, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE,
0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE, 0xFFFE,
0xFFFE, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC,
0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC, 0xFFFC,
0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8,
0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8, 0xFFF8, 0xFFF0, 0xFFF0,
0xFFF0, 0xFFF0, 0xFFF0, 0xFFF0, 0xFFF0, 0xFFF0, 0xFFF0, 0xFFF0,
0xFFF0, 0xFFF0, 0xFFE0, 0xFFE0, 0xFFE0, 0xFFE0, 0xFFE0, 0xFFE0,
0xFFE0, 0xFFE0, 0xFFE0, 0xFFE0, 0xFFE0, 0xFFC0, 0xFFC0, 0xFFC0,
0xFFC0, 0xFFC0, 0xFFC0, 0xFFC0, 0xFFC0, 0xFFC0, 0xFFC0, 0xFFC0,
0xFF80, 0xFF80, 0xFF80, 0xFF80, 0xFF80, 0xFF80, 0xFF80, 0xFF80,
0xFF80, 0xFF80, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00, 0xFF00,
0xFF00, 0xFF00, 0xFF00, 0xFE00, 0xFE00, 0xFE00, 0xFE00, 0xFE00,
0xFE00, 0xFE00, 0xFE00, 0xFE00, 0xFC00, 0xFC00, 0xFC00, 0xFC00,
0xFC00, 0xFC00, 0xFC00, 0xFC00, 0xFC00, 0xF800, 0xF800, 0xF800,
0xF800, 0xF800, 0xF800, 0xF800, 0xF800, 0xF800, 0xF800, 0xF000};

// --------  nastavenie hodinoveho signalu a PLL
_FOSCSEL(FNOSC_PRIPLL & IESO_ON);
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF  & POSCMD_HS);
_FICD(JTAGEN_OFF);


void set_modules(void) // globalne nastavenie pouzivanych modulov
{
				_T3MD = 1; // DISABLE timer3
				_T2MD = 0; // ENABLE timer2
				_T1MD = 1; // DISABLE timer1
				_I2C1MD = 1; //DISABLE i2c
				_U1MD = 1; // DISABLE uart1
				_SPI1MD = 0; //ENABLE spi1
				_AD1MD = 1; // DISABLE ad1
				_IC8MD = 1; // DISABLE input capture 8
				_IC7MD = 1; // DISABLE input capture 7
				_IC2MD = 1; // DISABLE input capture 2
				_IC1MD = 1; // DISABLE input capture 1
				_OC2MD = 1; // DISABLE output compare 2
				_OC1MD = 0; // ENABLE output compare 1 // pre PWM
}



void config_peripheral_pins(void) // nastavenie prekonfigurovatelnych pinov
{
__builtin_write_OSCCONL(OSCCON & ~(1<<6));

RPOR2bits.RP5R = 0b10010; //output compare 1 (PWM) RB5

RPOR6bits.RP13R = 0b01000; //SPI clock out to RP13
RPOR7bits.RP14R = 0b00111; //SPI data  out to RP14

__builtin_write_OSCCONL(OSCCON | (1<<6));
}

void ConfigPWM2(void)
{
	// Initialize Output Compare Module
	OC1CONbits.OCM = 0b000; // Disable Output Compare Module
	OC1R = 100; 			// Write the duty cycle for the first PWM pulse
	OC1RS = 200; 			// Write the duty cycle for the second PWM pulse
	OC1CONbits.OCTSEL = 0; 	// Select Timer 2 as output compare time base
	OC1R = 100; 			// Load the Compare Register Value
	OC1CONbits.OCM = 0b110; // Select the Output Compare mode

// Initialize and enable Timer2
	T2CONbits.TON = 0; // Disable Timer
	T2CONbits.TCS = 0; // Select internal instruction cycle clock
	T2CONbits.TGATE = 0; // Disable Gated Timer mode
	T2CONbits.TCKPS = 0b00; // Select 1:1 Prescaler

	TMR2 = 0x00; // Clear timer register
	PR2 = 0xFF; // Load the period value

//	IPC1bits.T2IP = 0x00; // Set Timer 2 Interrupt Priority Level
//	IFS0bits.T2IF = 0; // Clear Timer 2 Interrupt Flag
//	IEC0bits.T2IE = 0; // disable Timer 2 interrupt
	T2CONbits.TON = 1; // Start Timer
}

void ConfigInt0(void)
{
	_INT0IE = 1; // interrupt enabled 1 = enabled
	_INT0EP = 0; // interrupt on 1 = negative edge
	_INT0IP = 7; // priority 0=disbled  1=lowest 7=highest
}


void WriteSPI1(unsigned int data_out)
 {
     IFS0bits.SPI1IF = 0;       //clearing flag. This will be set, when the data
                                //is completly out from the SPI out buffer

     SPI1BUF = data_out; // & 0xff;    //  byte write

     while(IFS0bits.SPI1IF == 0);    // wait while all 8 bits are sent
     data_out = SPI1BUF;               //dummy read, avoiding overflow in internal buffer
 }



void ConfigSPI1(void)
{
			SPI1CON1bits.DISSCK = 0;	//SCK pin en
			SPI1CON1bits.DISSDO = 0;	//SDO pin	en
			SPI1CON1bits.MODE16 = 0;	//byte  mode
			SPI1CON1bits.SMP = 0;		//sample phase bit
			SPI1CON1bits.CKE = 1;		//clock edge select
			SPI1CON1bits.CKP = 0;		//clock polarity
			SPI1CON1bits.SSEN = 0;		//slave select dis
			SPI1CON1bits.MSTEN = 1;		//master en
			SPI1CON1bits.SPRE = 7;		//secondary prescaler 7 = 1:1
			SPI1CON1bits.PPRE = 2;		//primary prescaler 2 = 4:1

			SPI1CON2bits.FRMEN = 0;		//framing dis
			// if framing is disabled the next 3 register is not valid
			SPI1CON2bits.SPIFSD = 0;	//Frame Sync Pulse Direction Control
			SPI1CON2bits.FRMPOL = 1;	//Frame Sync Pulse Polarit
			SPI1CON2bits.FRMDLY = 0;	// Frame Sync Pulse Edge Select bit

			SPI1STATbits.SPISIDL = 0; // continue on idle mode
			SPI1STATbits.SPIROV = 0;	// flag clear
			SPI1STATbits.SPIEN = 1;  // enables SPI
}




void InitAdc(void) // settings for PCM 1850A ADC
{

SS_Adc = 0;
WriteSPI1(0x33);
WriteSPI1(0x0C);
SS_Adc = 1;

__delay_us(2);

SS_Adc = 0;
WriteSPI1(0x32);
WriteSPI1(0x02);
SS_Adc = 1;

__delay_us(2);

SS_Adc = 0;
WriteSPI1(0x31);
WriteSPI1(0x40 + 0x36);
SS_Adc = 1;

__delay_us(2);

SS_Adc = 0;
WriteSPI1(0x33);
WriteSPI1(0x4C);
SS_Adc = 1;

}

//----- z vstupnych hodnot vytvory vystup vhodny pre 2*12 led -------
void DrawLeds(unsigned int left, unsigned int right)
{

unsigned int buff;


	right = coeff_R[(right>>8)]; // nacitanie hodnoty z tabulky
	left = coeff_L[(left>>8)]; // nacitanie hodnoty z tabulky

// nacitanie z ramu je najrichlejsou metodou na nastavenie vhodnych led na vystupu
// dalo by sa aj mnohonasobnym IF, ktory vsak zobere viac vypocetneho casu,
// a naviac u roznych hodnot ma rozny cas prebehu, co je neziaduce.



	SS = 0; // aktivacia shift registerov

//---- je potrebne previest 2 hodnoty int na 3 hodnot 8 bitovych
//     hodnoty v int nikdy nemozu byt sirsie ako 12bit, preto je mozne ich previst


	SPI1BUF = (right>>4); // sending the first from the 3
	buff = SPI1BUF; //DUMMY READ spi buff  TO CLEAR STATUS FLAG

// medzi casom kym sa posiela prvych 8 bitov
// sa vypocita dalsich 8 bitov (2/3)
// je to trosku narocnejsia operacia a program by ajtak cakal na SPI,
// takze tuto sa uspori na case
	buff = 	(right & 0x000F)<<4   |   (left & 0x0F00)>>8;


	while(SPI1STATbits.SPIRBF == 0 ); // waiting for send the first 8 bits


	SPI1BUF = buff; // sending the 2/3 of 8 bits
	buff = SPI1BUF; //DUMMY READ spi buff  TO CLEAR STATUS FLAG

	while(SPI1STATbits.SPIRBF == 0 );// waiting for send the 2/3 of 8 bits


	SPI1BUF = left;// send the last 8 bits
	buff = SPI1BUF; //DUMMY READ spi buff  TO CLEAR STATUS FLAG

	while(SPI1STATbits.SPIRBF == 0 ); // waiting for send the 3/3 of 8 bits


	SS = 1; // konec prenosu dat, objavy sa nova hodnota na led bar

}


int main(void)
	{


//-------nastavenie PLL na generovanie pravneho kmitoctu
// --- core pobezi na 36.864 MHZ, co je 3 nasobok hlavneho hodinoveho signalu ADC

				PLLFBD=22; // M=24
				CLKDIVbits.PLLPOST=0; // N1=2
				CLKDIVbits.PLLPRE=2; // N2=4
				OSCTUN=7; // Tune FRC oscillator, if FRC is used
				RCONbits.SWDTEN=0; // Disable Watch Dog Timer

				while(OSCCONbits.LOCK!=1) {}; // Wait for PLL to lock

				RCONbits.SWDTEN=0;            /* Disable Watch Dog Timer*/



	_TRISB5 = 0;  //vystup na led
	_TRISB15 = 0; // vystup na SS 0 = out;
	_TRISA0 = 0; // vystup na SS_ADC  0 = out;


	SS = 1; // nastavenie inicialnych hodnot
	SS_Adc = 1; // nastavenie inicialnych hodnot


// nastavanie DSP core, aby fungovalo podla predstav

asm volatile ( "bset 0x044,#12"); // DSP multiply variables are unsigned //CORCON
asm volatile ( "bset 0x044,#0"); //set DSP multiply to integer , not fractional

asm volatile ( "bclr 0x044,#7"); //saturation disabled in ACCa
asm volatile ( "bclr 0x044,#6"); //saturation disabled in ACCb
asm volatile ( "bset 0x044,#4"); //super satur enable => use all 40 bit of all ACC


// inicializacia vsetkych periferii
			  	set_modules();
				config_peripheral_pins();
				ConfigSPI1();
				InitAdc();
 				ConfigPWM2();
				__delay_us(100);
				ConfigInt0();



while(1)
{


// refresh LED grafu za kazdych 480 cyklov, tj 48000/480 = 100Hz
if (cnt > 480)
	{
	_INT0IE = 0; // interrupt disabled, pretoze generovanie a posileanie
                 // dat na led display trva viac ako 1 cyklus.

 	DrawLeds(L_out,R_out); // vykreslenie novych hodnot

	cnt = 0; // nulovanie pocitadla

    _INT0IF = 0; // vynulovanie int flagu, aby nahodov nezacal zbierat data v strede vstupneho slova
	_INT0IE = 1; // interrupt enabled
	}
/*
asm volatile ( "NOP");
asm volatile ( "NOP");
asm volatile ( "NOP");
asm volatile ( "NOP");
asm volatile ( "NOP");
*/
}

return 0;
}



//interrupt na L/R clock, external interrupt.
// toto prerusenie neuklada ziadne pracovne registre, anie stavovy register, ulozi len navratovu hodnotu.
// je to potrebne preto, aby pri interrupu program stihol zaznamenat prvy datovy bit
// u tohoto prerusenia nieje ani manualne spravene ukladanie registrov, co vsak v tomto pripade nevadi,
// pretoze pouzivam len volne registre s ktorimi prekladac nepracuje bezne, alebo ich pouziva ako univerzalne a ajtak by sa neukladali.
// overene debuggingom a simulaciou, treba VYPNUT OPTIMALIZACIU !!!!! inak kod nepracuje podla napisanich a ZRUTI sa.

// v interrupte sa dekoduje I2S protokol z A/D prevodniku. Najprv lava potom prava stana.
// Prevodnik pracuje s 24 bitovou sirkou dat, na vizualizaciu hojne postaci 16 bitov, takze aj dekodovanie je len na 16bit na kanal.
// po dekodovani aktualnej hodnoty je ulozena 15 bitova absolutna hodnota signalu, a 8 bitova reprezentacia toho isteho signalu, na PWM ledku

// nakoniec nova hodnota je spriemernena  filtrom s exponencialnym starnutim vzorkov, ktory ma simulovat vybijanie RC clanku.
// Pretoze vzorkovacia frekvencia je 48kHz, bolo potreba mat dost mali a presny koeficient starnutia, ktora bude v tomto pripade
// 65532/65536 = 0.999939  Pretoze pocitanie s float by bolo narocne, vyuzije sa DSP coreu, a 40 bitoveho akumulatoru,
// kde je mozno signal s dostatocnou presnostou spracovat. Signal sa vynasoby 65532, potom sa vydeli 65536 (posunie o 16b doprava),
// tym za par hodinovych taktov mame signal vynasobeny potrebnym koeficientom

// exponenciane starnutie je mozno zapoisat nasledovne :
// x_next = (x_previous + x_actual) * coeff;

// coeff je od < 0 1 >
// x_next je vystup filtru a v nasledujucum taktu bude z neho x_previous
// x_previous je predosli vysledok
// x_actual je aktualna hodnota (absolutna hodnota z A/D prevodniku)

// poznamka: exponencialne stranutie pracuje len na "vybijanie", to znamena,
// ze hodnota vo filtru je vacsia ako vsupna. Ked je hodnota vstupna vacsia ako vo filtru,
// okamzite sa prepise hodnota na vacsiu.


//nakoniec vsetkeho na ledku na doske sa posle vacsia hodnota, vystup je PWM, tym padom led zmeni jas na silu signalu (hudby)

//cela nasledujuca cast je napisana tak, aby pri vetveni programu trvali rozne vetve stejni cas.
// Kompenzuje sa na najdlchsi proces, preto je u viacerich kodov niekolko NOP instrukcii

void __attribute__((__interrupt__ , __no_auto_psv__)) _INT0Interrupt(void)
{
asm volatile ( "clr w0"); // vynulovanie w0 registra, v tomto bude hodnota dekodovana z I2S

// kompenzacia casu, aby sme sa trafili do sterdu stredu bitu
asm volatile ( "REPEAT #5"); //zopakuje nasledujucu instrukciu n krat // trva 2 + este instrukcia za nim * n
asm volatile ( "NOP");


// instrukcie led on a led off boli potrebne na debugging a na meranie casovania, teraz sluzia ako instrukci NOP,
// pretoze vystup na LEd diodu teraz uz riadi otput compare 2, cize PWM.
// dalo by sa prepisat na nop, kazdopadne je zbytocne , na funkcnosti nic nezmeni
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8"); //ked na vstupnom pinu je 1 tak nastavy bit MSB (15) v W0 do 1
asm volatile ( "bset w0,#15"); // ked nie tak tuto instrukciu preskoci a zostane tam nula

// v dalsich nasledujucich 14 blokoch sa toto opakuje dookola, akurat sa nastavuje prislusny bit vo w0 => 14 13 12 .... 1 az 0
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#14");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#13");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#12");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#11");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#10");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#9");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#8");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#7");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#6");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#5");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#4");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#3");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#2");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#1");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#0");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
//**************************************************************

// v tomto bode W0 obsahuje znamienkovu hodnotu signalu

WREG6 = WREG0; // prekopirovanie hodnoty do w6 (DSP nepracuje s w0)

if (WREG6 & 0x8000) WREG6 = ~WREG6 +1; //absolutna hodnota
else
	{
	    asm volatile ("NOP"); //kompenzacia casu
        asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP"); //kompenzacia casu
	}

//PWM LED
L_led = WREG6>>8; // orezana (8b msb <= 16b) absolutna hodnota sa ulozi pre PWM led

// ACCa akumulator a z DSP core obsahuje aktualnu hodnotu filtru
// w6 = abs(input signal)
// w7 koeficient na nasobenie
// w5 ot nem nem szabad kinullazni az a belso buffer az egesz reszre

	if (WREG6 > L_out)  // WREG6 ba ugyis ADC_L van
        {
        L_out = WREG6;

		asm volatile ("NOP"); //kompenzaci a casu
        asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP"); //kompenzaci a casu

		}
    else
        {
		WREG5 = L_out;

/*
aby bolo mozne presne pocitat, je treba vtupne hodnoty este vynasobit.
Hodnota s ktorou sa ma vynasobit zavisi od koeficientu (akysi digitalne tau).

       cas tau v      koeficient                cas tau pri Fs= 48000Hz
     poctu vzorkou                kompenzacia

2^10 	1024	        65472 	      64                21.334ms
2^11 	2048	        65504 	      32                42.667ms
2^12 	4096	        65520	      16                85.334ms
2^13 	8192	        65528	      8                 170.667ms
2^14 	16384	        65532	      4                 341.334ms
2^15 	32768	        65534	      2                 682.667ms
2^16 	65536	        65535	      1                 1.365s
*/
		//vynulovanie celej casti bufferu pomocou registru
		asm volatile ("clr w7");
		//tym padom zostane v nom len zbytek z predosleho cyklu
		asm volatile ("mov w7,0x24");

        //kompenzacny koeficient aj s truncation vstupneho signalu
		asm volatile ("mov #4,w7");
		//vynasobim vstupny signal koeficientom aby filter pracoval s celou 32b sirkou
		asm volatile ("mac w6*w7,A");

        // koeficient casu
		asm volatile ("mov #65532,w7");
		// vynasobim predoslu hodnotu a pridam zbytek z predosleho cyklu
		asm volatile ("mac w5*w7,A");
        // vysledok ulozim (len msb 16b)
		asm volatile ("sac A,w5");

		L_out = WREG5;
		}

//*kompenzacia casu na druhy kanal
asm volatile ( "REPEAT #149");
asm volatile ( "NOP");
//*********************************************************************************************************
//***************************************right channel******************************************************
//*********************************************************************************************************
asm volatile ( "clr w0");

// caka viac ako u predosleho kanalu, pretoze nemusi do interruptu vojst
asm volatile ( "REPEAT #16");
asm volatile ( "NOP");

// inak odtialto je to totozne s lavim kanalom, preto komenty budu len niekde

asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#15");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#14");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#13");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#12");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#11");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#10");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#9");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#8");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#7");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#6");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#5");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#4");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#3");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#2");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#1");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
asm volatile ( "REPEAT #6");
asm volatile ( "NOP");
//**************************************************************
asm volatile ( "bset.b 0x02ca,#5"); //led on
//******
asm volatile ( "BTSC 0x02CA,#8");
asm volatile ( "bset w0,#0");
//*******
asm volatile ( "bclr.b 0x02ca,#5"); // led off
//**********
//**************************************************************

WREG6 = WREG0;

if (WREG6 & 0x8000) WREG6 = ~WREG6 +1; //absolutna hodnota
else
	{
	    asm volatile ("NOP"); //ido kiegyenlites
        asm volatile ("NOP"); //ido kiegyenlites
		asm volatile ("NOP"); //ido kiegyenlites
		asm volatile ("NOP"); //ido kiegyenlites
		asm volatile ("NOP"); //ido kiegyenlites
	}

//PWM LED
R_led = WREG6>>8; // absolutnu hodnotu (orezanu ) do bufferu PWM



// ACCb akumulator a z DSP core obsahuje aktualnu hodnotu filtru
// w6 = abs(input signal)
// w7 koeficient na nasobenie
// w5 ot nem nem szabad kinullazni az a belso buffer az egesz reszre

	if (WREG6 > R_out)  // WREG6 obsahuje ajtak hodnotu ADC_L
        {
        R_out = WREG6;

		asm volatile ("NOP"); //kompenzacia casu
        asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP");
		asm volatile ("NOP"); //kompenzacia casu

		}
    else
        {
		WREG5 = R_out;

/*
aby bolo mozne presne pocitat, je treba vtupne hodnoty este vynasobit.
Hodnota s ktorou sa ma vynasobit zavisi od koeficientu (akysi digitalne tau).

       cas tau v      koeficient                cas tau pri Fs= 48000Hz
     poctu vzorkou                kompenzacia

2^10 	1024	        65472 	      64                21.334ms
2^11 	2048	        65504 	      32                42.667ms
2^12 	4096	        65520	      16                85.334ms
2^13 	8192	        65528	      8                 170.667ms
2^14 	16384	        65532	      4                 341.334ms
2^15 	32768	        65534	      2                 682.667ms
2^16 	65536	        65535	      1                 1.365s
*/

        //vynulovanie celej casti bufferu pomocou registru
		asm volatile ("clr w7");
		//tym padom zostane v nom len zbytek z predosleho cyklu
		asm volatile ("mov w7,0x2A");

        //kompenzacny koeficient aj s truncation vstupneho signalu
		asm volatile ("mov #4,w7");
		//vynasobim vstupny signal koeficientom aby filter pracoval s celou 32b sirkou
		asm volatile ("mac w6*w7,B");

        // koeficient casu
		asm volatile ("mov #65532,w7");
		// vynasobim predoslu hodnotu a pridam k zbytku + skalovaneho vstupu z predosleho cyklu
		asm volatile ("mac w5*w7,B");
        // vysledok ulozim (len msb 16b)
		asm volatile ("sac B,w5");

		R_out = WREG5;
		}

//***************************od tohoto bodu neni potreba kompenzacia casu*******************

//PWM na led
if (R_led > L_led) OC1RS = R_led;
else OC1RS = L_led;


cnt++;

_INT0IF = 0; // nulovanie interrupt falgu
};
